# Copyright © 2022 Arrikto Inc.  All Rights Reserved.

"""Kale SDK.

This script uses an ML pipeline to train and serve an LightGBM Model.
"""

import lightgbm as lgb

from typing import Tuple

from sklearn.model_selection import train_test_split
from sklearn.datasets import load_iris

from kale.serve import serve
from kale.ml import Signature
from kale.types import MarshalData
from kale.sdk import pipeline, step
from kale.common import mlmdutils, artifacts


@step(name="data_loading")
def load_split_dataset() -> Tuple[MarshalData, MarshalData]:
    """Fetch Iris dataset."""
    # get data and target of the dataset
    iris = load_iris()
    x = iris.data
    y = iris.target

    # split the dataset
    x_train, _, y_train, _ = train_test_split(x, y, test_size=.2,
                                              random_state=42)
    return x_train, y_train


@step(name="model_training")
def train(x: MarshalData, y: MarshalData) -> MarshalData:
    """Train a Booster model."""
    lgb_train = lgb.Dataset(x, y)

    params = {"objective": "multiclass",
              "metric": "softmax",
              "num_class": 3}

    model = lgb.train(params=params, train_set=lgb_train)

    return model


@step(name="register_model")
def register_model(model: MarshalData, x: MarshalData, y: MarshalData) -> int:
    """Register the model in the MLMD store."""
    mlmd = mlmdutils.get_mlmd_instance()

    signature = Signature(
        input_size=[1] + list(x[0].shape),
        output_size=[1] + list(y[0].shape),
        input_dtype=x.dtype,
        output_dtype=y.dtype)

    model_artifact = artifacts.LightGBMModel(
        model=model,
        description="A simple LightGBM classifier",
        version="1.0.0",
        author="Kale",
        signature=signature,
        tags={"app": "lightgbm-tutorial"}).submit_artifact()

    mlmd.link_artifact_as_output(model_artifact.id)
    return model_artifact.id


@step(name="serve_model")
def serve_model(model_id: int):
    serve(name="lightgbm-tutorial", model_id=model_id)


@pipeline(name="regression", experiment="lightgbm-tutorial")
def ml_pipeline():
    """Run the ML pipeline."""
    x_train, y_train = load_split_dataset()
    model = train(x_train, y_train)
    model_id = register_model(model, x_train, y_train)
    serve_model(model_id)


if __name__ == "__main__":
    ml_pipeline()
